import { ActionContext } from 'vuex'
import { BareActionContext, getStoreBuilder } from 'vuex-typex'

import { mutations as mut } from './mutations'
import { Authentication } from '@/store/modules/authentication'
import { SubmissionNotes } from '@/store/modules/submission-notes'
import * as api from '@/api'
import router from '@/router/index'
import { RootState } from '@/store/store'
import { FeedbackTable } from '@/store/modules/feedback_list/feedback-table'
import { Subscriptions } from '@/store/modules/subscriptions'

async function getExamTypes(context: BareActionContext<RootState, RootState>) {
  const examTypes = await api.fetchExamTypes({})
  mut.SET_EXAM_TYPES(examTypes)
}
async function updateSubmissionTypes(
  context: BareActionContext<RootState, RootState>,
) {
  const submissionTypes = await api.fetchSubmissionTypes()
  submissionTypes.forEach(type => {
    mut.UPDATE_SUBMISSION_TYPE(type)
  })
}
async function getStudents(
  context: BareActionContext<RootState, RootState>,
  opt: { studentPks: Array<string>} = {
    studentPks: []
  }
) {
  if (opt.studentPks.length === 0) {
    const students = await api.fetchAllStudents()
    mut.SET_STUDENTS(students)
    return students
  } else {
    const students = await Promise.all(
      opt.studentPks.map((pk: string) =>
        api.fetchStudent({pk})
      )
    )
    students.forEach(student => mut.SET_STUDENT(student))
    return students
  }
}
async function getTutors() {
  const tutors = await api.fetchAllTutors()
  mut.SET_TUTORS(tutors)
}
async function getFeedback(
  context: BareActionContext<RootState, RootState>,
  fetchFeedbackArg: { ofSubmission: string }
) {
  const feedback = await api.fetchFeedback(fetchFeedbackArg)
  mut.SET_FEEDBACK(feedback)
  return feedback
}
async function getSubmissionFeedbackTest(
  context: BareActionContext<RootState, RootState>,
  submissionPkObj: { pk: string }
) {
  const submission = await api.fetchSubmissionFeedbackTests(submissionPkObj)
  mut.SET_SUBMISSION(submission)
}
async function getStatistics() {
  const statistics = await api.fetchStatistics()
  mut.SET_STATISTICS(statistics)
}
function logout(
  context: BareActionContext<RootState, RootState>,
  message = ""
) {
  // logout actually receives a normal ActionContext, but for some reason vuex-typex mandates a BareActionContext
  // TODO there has to be a better way
  const { commit, getters, state } = <ActionContext<RootState, RootState>>(
    context
  )
  if (Authentication.isStudent) {
    // change active to false when user logs out
    // TODO this should belong in auth module
    api.changeActiveForUser(Authentication.state.user.pk, false)
  }
  mut.RESET_STATE()
  FeedbackTable.RESET_STATE()
  Authentication.RESET_STATE()
  Subscriptions.RESET_STATE()
  SubmissionNotes.RESET_STATE()
  Authentication.SET_MESSAGE(message)
  router.push({ name: "login" })
  router.go(0)
}

const mb = getStoreBuilder<RootState>()

export const actions = {
  updateSubmissionTypes: mb.dispatch(updateSubmissionTypes),
  getExamTypes: mb.dispatch(getExamTypes),
  getStudents: mb.dispatch(getStudents),
  getTutors: mb.dispatch(getTutors),
  getFeedback: mb.dispatch(getFeedback),
  getSubmissionFeedbackTest: mb.dispatch(getSubmissionFeedbackTest),
  getStatistics: mb.dispatch(getStatistics),
  logout: mb.dispatch(logout)
}
